"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connection = void 0;
const axios_1 = __importDefault(require("axios"));
const url_1 = require("url");
const form_data_1 = __importDefault(require("form-data"));
const fs_1 = __importDefault(require("fs"));
const authorizeUrl = 'https://dah2vb2cprod.b2clogin.com/914d88b1-3523-4bf6-9be4-1b96b4f6f919/oauth2/v2.0/token?p=B2C_1A_signup_signin_common';
const baseApiUrl = 'https://api.mps.ford.com/api/fordconnect';
const vehiclesUrl = '/v2/vehicles';
const vehicleStatusUrl = '/v1/vehicles/{vehicleId}/status';
const vehicleSatusRefreshUrl = '/v1/vehicles/{vehicleId}/statusrefresh/{commandId}';
const vehicleUnlockUrl = '/v1/vehicles/{vehicleId}/unlock';
const vehicleUnlockRefreshUrl = '/v1/vehicles/{vehicleId}/unlock/{commandId}';
const vehicleLockUrl = '/v1/vehicles/{vehicleId}/lock';
const vehicleLockRefreshUrl = '/v1/vehicles/{vehicleId}/lock/{commandId}';
const vehicleStartUrl = '/v1/vehicles/{vehicleId}/startEngine';
const vehicleStartRefreshUrl = '/v1/vehicles/{vehicleId}/start/{commandId}';
const vehicleStopUrl = '/v1/vehicles/{vehicleId}/stop';
const vehicleStopRefreshUrl = '/v1/vehicles/{vehicleId}/stop/{commandId}';
const vehicleStartChargeUrl = '/v1/vehicles/{vehicleId}/startCharge';
const vehicleStartChargeRefreshUrl = '/v1/vehicles/{vehicleId}/startCharge/{commandId}';
const vehicleStopChargeUrl = '/v1/vehicles/{vehicleId}/stopCharge';
const vehicleStopChargeRefreshUrl = '/v1/vehicles/{vehicleId}/stopCharge/{commandId}';
const vehicleInformationUrl = '/v3/vehicles/{vehicleId}';
const headers = {
    'User-Agent': 'Mozilla/5.0',
    'Accept-Language': 'en-US,en;q=0.9',
    'Accept-Encoding': 'gzip, deflate, br',
};
class Connection {
    constructor(config, log, api) {
        this.config = config;
        this.log = log;
        this.api = api;
        this.credentials = this.getCredentials();
    }
    getVehicles() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.credentials.access_token) {
                yield this.auth();
            }
            const options = {
                method: 'GET',
                url: baseApiUrl + vehiclesUrl,
                headers: {
                    'Content-Type': 'application/json',
                    'Application-Id': this.config.application_id,
                    Authorization: `Bearer ${this.credentials.access_token}`,
                },
            };
            try {
                const result = yield (0, axios_1.default)(options);
                if (result.status < 300 && result.data) {
                    const vehicles = [];
                    for (const info of result.data.vehicles) {
                        vehicles.push(Object.assign(Object.assign({}, info), result.data.vehicles[0]));
                    }
                    return vehicles;
                }
                return [];
            }
            catch (error) {
                this.log.error(`Error occurred during request: ${error.message}`);
                if (error.response) {
                    this.log.error(`Response status: ${error.response.status}`);
                    this.log.error(`Response data: ${JSON.stringify(error.response.data)}`);
                    this.log.error(`Response headers: ${JSON.stringify(error.response.headers)}`);
                }
                else if (error.request) {
                    this.log.error(`Request made but no response received: ${error.request}`);
                }
                else {
                    this.log.error(`Error details: ${JSON.stringify(error)}`);
                }
                return [];
            }
        });
    }
    getVehicleInformation(vehicleId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.credentials.access_token) {
                yield this.auth();
            }
            const url = baseApiUrl + vehicleInformationUrl.replace('{vehicleId}', vehicleId);
            const options = {
                method: 'GET',
                url: url,
                headers: {
                    'Content-Type': 'application/json',
                    'Application-Id': this.config.application_id,
                    Authorization: `Bearer ${this.credentials.access_token}`,
                },
            };
            try {
                const result = yield axios_1.default.request(options);
                if (result.status < 300 && result.data) {
                    if (result.data.status === 'SUCCESS') {
                        return result.data.vehicle;
                    }
                    return {};
                }
                else {
                    this.log.error(`Vehicle info failed with status: ${result.status}`);
                }
                return {};
            }
            catch (error) {
                this.log.error(`Vehicle info failed with error: ${error.code || error.response.status}`);
                return {};
            }
        });
    }
    issueCommand(vehicleId, command) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.credentials.access_token) {
                yield this.auth();
            }
            let url = '';
            switch (command) {
                case 'unlock':
                    url = vehicleUnlockUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'lock':
                    url = vehicleLockUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'start':
                    url = vehicleStartUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'stop':
                    url = vehicleStopUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'startCharge':
                    url = vehicleStartChargeUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'stopCharge':
                    url = vehicleStopChargeUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'status':
                    url = vehicleStatusUrl.replace('{vehicleId}', vehicleId);
                    break;
                default:
                    break;
            }
            const options = {
                method: 'POST',
                url: baseApiUrl + url.replace('{vehicleId}', vehicleId),
                headers: {
                    'Content-Type': 'application/json',
                    'Application-Id': this.config.application_id,
                    Authorization: `Bearer ${this.credentials.access_token}`,
                },
            };
            try {
                const result = yield axios_1.default.request(options);
                if (result.status < 300 && result.data) {
                    return result.data;
                }
                else {
                    this.log.error(`Command failed with status: ${JSON.stringify(result)}`);
                }
                return {};
            }
            catch (error) {
                this.log.error(`Command failed with error: ${JSON.stringify(error)}`);
                return {};
            }
        });
    }
    issueCommandRefresh(commandId, vehicleId, command) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.credentials.access_token) {
                yield this.auth();
            }
            let url = '';
            switch (command) {
                case 'unlock':
                    url = vehicleUnlockRefreshUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'lock':
                    url = vehicleLockRefreshUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'start':
                    url = vehicleStartRefreshUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'stop':
                    url = vehicleStopRefreshUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'startCharge':
                    url = vehicleStartChargeRefreshUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'stopCharge':
                    url = vehicleStopChargeRefreshUrl.replace('{vehicleId}', vehicleId);
                    break;
                case 'status':
                    url = vehicleSatusRefreshUrl.replace('{vehicleId}', vehicleId);
                    break;
                default:
                    break;
            }
            const options = {
                method: 'GET',
                url: baseApiUrl + url.replace('{commandId}', commandId),
                headers: {
                    'Content-Type': 'application/json',
                    'Application-Id': this.config.application_id,
                    Authorization: `Bearer ${this.credentials.access_token}`,
                },
            };
            try {
                const result = yield axios_1.default.request(options);
                if (result.status < 300 && result.data) {
                    let commandStatus = result.data.commandStatus;
                    let tries = 10;
                    while (commandStatus === 'QUEUED' && tries > 0) {
                        yield new Promise((resolve) => setTimeout(resolve, 5000));
                        const refreshResult = yield (0, axios_1.default)(options);
                        commandStatus = refreshResult.data.commandStatus;
                        tries--;
                    }
                    return result.data;
                }
                else {
                    this.log.error(`Command failed with status: ${JSON.stringify(result)}`);
                }
                return {};
            }
            catch (error) {
                if (error.response) {
                    this.log.error(`issueCommandRefresh Response status: ${error.response.status}`);
                    this.log.error(`issueCommandRefresh Response data: ${JSON.stringify(error.response.data)}`);
                    this.log.error(`issueCommandRefresh Response headers: ${JSON.stringify(error.response.headers)}`);
                }
                else if (error.request) {
                    this.log.error(`issueCommandRefresh Request made but no response received: ${error.request}`);
                }
                else {
                    this.log.error(`issueCommandRefresh Error details: ${JSON.stringify(error)}`);
                }
                return {};
            }
        });
    }
    auth() {
        return __awaiter(this, void 0, void 0, function* () {
            this.credentials = this.getCredentials();
            if (!this.config.code && !this.config.client_secret) {
                this.log.error('Missing code or client_secret');
                return;
            }
            if (!this.credentials.refresh_token) {
                return yield this.getAccessToken();
            }
            else {
                const refreshToken = yield this.getRefreshToken();
                return refreshToken;
            }
        });
    }
    getAccessToken() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.config.code || !this.config.client_secret || !this.config.client_id) {
                    this.log.error('Missing code or client_secret, or client_id');
                    return;
                }
                const data = new url_1.URLSearchParams();
                data.append('grant_type', 'authorization_code');
                data.append('client_id', this.config.client_id);
                data.append('client_secret', this.config.client_secret);
                data.append('code', this.config.code);
                data.append('redirect_url', 'https://localhost:3000');
                const options = {
                    method: 'post',
                    url: authorizeUrl,
                    headers: Object.assign({ 'Content-Type': 'application/x-www-form-urlencoded' }, headers),
                    maxBodyLength: Infinity,
                    data: data.toString(),
                };
                const res = yield axios_1.default.request(options);
                if (res.status === 200 && res.data.access_token) {
                    this.log.debug('Successfully got token from FordPass API');
                    this.credentials.refresh_token = res.data.refresh_token;
                    this.credentials.access_token = res.data.access_token;
                    this.credentials.expires_in = res.data.expires_in;
                    return this.getRefreshToken();
                }
            }
            catch (error) {
                this.log.error("Auth failed for FordPass.  Please follow the FordPass API Setup instructions to retrieve the 'code'.");
                this.log.error(`Error occurred during request: ${error.message}`);
                if (error.response) {
                    this.log.error(`getAccessToken Response status: ${error.response.status}`);
                    this.log.error(`getAccessToken Response data: ${JSON.stringify(error.response.data)}`);
                    this.log.error(`getAccessToken Response headers: ${JSON.stringify(error.response.headers)}`);
                }
                else if (error.request) {
                    this.log.error(`Request made but no response received: ${error.request}`);
                }
                else {
                    this.log.error(`getAccessToken Error details: ${JSON.stringify(error)}`);
                }
            }
        });
    }
    getRefreshToken() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const data = new form_data_1.default();
                data.append('grant_type', 'refresh_token');
                data.append('refresh_token', this.credentials.refresh_token);
                data.append('client_id', this.config.client_id);
                data.append('client_secret', this.config.client_secret);
                const options = {
                    method: 'post',
                    maxBodyLength: Infinity,
                    url: authorizeUrl,
                    headers: Object.assign({ 'Content-Type': 'multipart/form-data' }, data.getHeaders()),
                    data: data,
                };
                const res = yield axios_1.default.request(options);
                if (res.status === 200 && res.data.access_token) {
                    this.credentials.access_token = res.data.access_token;
                    this.credentials.refresh_token = res.data.refresh_token;
                    this.credentials.expires_in = res.data.expires_in;
                    this.createOrUpdateCredentials();
                    return res.data;
                }
                else {
                    this.log.error(`Auth failed with status: ${res.status}`);
                }
            }
            catch (error) {
                if (error.response) {
                    this.log.error(`getRefreshToken Response status: ${error.response.status}`);
                    this.log.error(`getRefreshToken Response data: ${JSON.stringify(error.response.data)}`);
                    this.log.error(`getRefreshToken Response headers: ${JSON.stringify(error.response.headers)}`);
                }
                else if (error.request) {
                    this.log.error(`getRefreshToken Request made but no response received: ${error.request}`);
                }
                else {
                    this.log.error(`getRefreshToken Error details: ${JSON.stringify(error)}`);
                }
            }
        });
    }
    createOrUpdateCredentials() {
        return __awaiter(this, void 0, void 0, function* () {
            const storagePath = this.api.user.storagePath();
            const credentialsPath = `${storagePath}/fordpass-security-credentials.json`;
            if (!fs_1.default.existsSync(credentialsPath)) {
                fs_1.default.writeFileSync(credentialsPath, JSON.stringify(this.credentials));
            }
            else {
                fs_1.default.writeFileSync(credentialsPath, JSON.stringify(this.credentials));
            }
            return this.getCredentials();
        });
    }
    getCredentials() {
        try {
            const storagePath = this.api.user.storagePath();
            const credentialsPath = `${storagePath}/fordpass-security-credentials.json`;
            if (fs_1.default.existsSync(credentialsPath)) {
                const credentials = JSON.parse(fs_1.default.readFileSync(credentialsPath).toString());
                return credentials;
            }
        }
        catch (error) {
            this.log.error(`getCredentials Error occurred retrieving credentials: ${error}`);
        }
        return { access_token: '', refresh_token: '', expires_in: 0 };
    }
}
exports.Connection = Connection;
//# sourceMappingURL=fordpass-connection.js.map