"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Vehicle = void 0;
const vehicle_1 = require("./types/vehicle");
const fordpass_connection_1 = require("./fordpass-connection");
const events_1 = require("events");
class Vehicle extends events_1.EventEmitter {
    constructor(name, vehicleId, config, log, api) {
        super();
        this.updating = false;
        this.config = config;
        this.log = log;
        this.api = api;
        this.name = name;
        this.vehicleId = vehicleId;
        this.autoRefresh = config.autoRefresh || false;
        this.refreshRate = config.refreshRate || 180;
    }
    issueCommand(command) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.updating) {
                return '';
            }
            this.updating = true;
            let commandType = '';
            switch (command) {
                case vehicle_1.Command.START: {
                    commandType = 'remoteStart';
                    break;
                }
                case vehicle_1.Command.STOP: {
                    commandType = 'stop';
                    break;
                }
                case vehicle_1.Command.LOCK: {
                    commandType = 'lock';
                    break;
                }
                case vehicle_1.Command.UNLOCK: {
                    commandType = 'unlock';
                    break;
                }
                case vehicle_1.Command.REFRESH: {
                    commandType = 'status';
                    break;
                }
                default: {
                    this.log.error('invalid command');
                    break;
                }
            }
            if (commandType) {
                try {
                    const result = yield new fordpass_connection_1.Connection(this.config, this.log, this.api).issueCommand(this.vehicleId, commandType);
                    if (result) {
                        if (command !== vehicle_1.Command.REFRESH) {
                            yield new fordpass_connection_1.Connection(this.config, this.log, this.api).issueCommand(this.vehicleId, 'status');
                        }
                        this.updating = false;
                        return result.commandId;
                    }
                }
                catch (error) {
                    this.updating = false;
                    if (error.response) {
                        this.log.error(`Response status: ${error.response.status}`);
                        this.log.error(`Response data: ${JSON.stringify(error.response.data)}`);
                        this.log.error(`Response headers: ${JSON.stringify(error.response.headers)}`);
                    }
                    else if (error.request) {
                        this.log.error(`Request made but no response received: ${error.request}`);
                    }
                    else {
                        this.log.error(`Error details: ${JSON.stringify(error)}`);
                    }
                }
            }
            return '';
        });
    }
    issueCommandRefresh(commandId, command) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.updating) {
                return '';
            }
            this.updating = true;
            let commandType = '';
            switch (command) {
                case vehicle_1.Command.START: {
                    commandType = 'remoteStart';
                    break;
                }
                case vehicle_1.Command.STOP: {
                    commandType = 'stop';
                    break;
                }
                case vehicle_1.Command.LOCK: {
                    commandType = 'lock';
                    break;
                }
                case vehicle_1.Command.UNLOCK: {
                    commandType = 'unlock';
                    break;
                }
                case vehicle_1.Command.REFRESH: {
                    commandType = 'status';
                    break;
                }
                default: {
                    this.log.error('invalid command');
                    break;
                }
            }
            if (commandType) {
                try {
                    const result = yield new fordpass_connection_1.Connection(this.config, this.log, this.api).issueCommandRefresh(commandId, this.vehicleId, commandType);
                    if (result) {
                        this.updating = false;
                        return result;
                    }
                }
                catch (error) {
                    this.updating = false;
                    if (error.response) {
                        this.log.error(`issueCommandRefresh Response status: ${error.response.status}`);
                        this.log.error(`issueCommandRefresh Response data: ${JSON.stringify(error.response.data)}`);
                        this.log.error(`issueCommandRefresh Response headers: ${JSON.stringify(error.response.headers)}`);
                    }
                    else if (error.request) {
                        this.log.error(`issueCommandRefresh Request made but no response received: ${error.request}`);
                    }
                    else {
                        this.log.error(`issueCommandRefresh Error details: ${JSON.stringify(error)}`);
                    }
                }
            }
            return '';
        });
    }
    retrieveVehicleInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield new fordpass_connection_1.Connection(this.config, this.log, this.api).getVehicleInformation(this.vehicleId);
            if (result) {
                this.log.debug(`Retrieved vehicle information for vehicle.  Mileage: ${result.vehicleDetails.mileage} Distance to E: ${result.vehicleDetails.batteryChargeLevel.distanceToEmpty * 0.621371} Tire Pressure Warning: ${result.vehicleStatus.tirePressureWarning}`);
                this.info = result;
                this.vehicleId = result.vehicleId;
                this.name = result.nickName;
            }
            return;
        });
    }
}
exports.Vehicle = Vehicle;
//# sourceMappingURL=fordpass.js.map