"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const fordpass_1 = require("./fordpass");
const vehicle_1 = require("./types/vehicle");
const fordpass_connection_1 = require("./fordpass-connection");
const accessory_1 = require("./accessory");
let hap;
let Accessory;
const PLUGIN_NAME = 'homebridge-fordpass';
const PLATFORM_NAME = 'FordPass';
class FordPassPlatform {
    constructor(log, config, api) {
        this.accessories = [];
        this.pendingLockUpdate = false;
        this.pendingStatusUpdate = false;
        this.log = log;
        this.api = api;
        this.config = config;
        if (!config) {
            return;
        }
        if (!config.code || !config.client_secret) {
            this.log.error('Please add a code and client_secret to your config.json.  See README.md for more information.');
            return;
        }
        api.on("didFinishLaunching", this.didFinishLaunching.bind(this));
    }
    configureAccessory(accessory) {
        this.log.info(`Configuring accessory ${accessory.displayName}`);
        accessory.on("identify", () => {
            this.log.info(`${accessory.displayName} identified!`);
        });
        this.vehicle = new fordpass_1.Vehicle(accessory.context.name, accessory.context.vin, this.config, this.log, this.api);
        const fordAccessory = new accessory_1.FordpassAccessory(accessory);
        const defaultState = hap.Characteristic.LockTargetState.UNSECURED;
        const lockService = fordAccessory.createService(hap.Service.LockMechanism);
        const switchService = fordAccessory.createService(hap.Service.Switch);
        const batteryService = fordAccessory.createService(hap.Service.Battery, this.config.batteryName || 'Fuel Level');
        if (this.config.chargingSwitch) {
            fordAccessory.createService(hap.Service.OccupancySensor, 'Charging');
        }
        else {
            fordAccessory.removeService(hap.Service.OccupancySensor, 'Charging');
        }
        if (this.config.plugSwitch) {
            fordAccessory.createService(hap.Service.OccupancySensor, 'Plug');
        }
        else {
            fordAccessory.removeService(hap.Service.OccupancySensor, 'Plug');
        }
        lockService.setCharacteristic(hap.Characteristic.LockCurrentState, defaultState);
        lockService
            .setCharacteristic(hap.Characteristic.LockTargetState, defaultState)
            .getCharacteristic(hap.Characteristic.LockTargetState)
            .on("set", (value, callback) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j;
            this.log.debug(`
          SET ${value ? 'Locking' : 'Unlocking'} ${accessory.displayName} 
          ${(_c = (_b = (_a = this.vehicle) === null || _a === void 0 ? void 0 : _a.info) === null || _b === void 0 ? void 0 : _b.vehicleStatus.lockStatus) === null || _c === void 0 ? void 0 : _c.value}`);
            if (value === (((_f = (_e = (_d = this.vehicle) === null || _d === void 0 ? void 0 : _d.info) === null || _e === void 0 ? void 0 : _e.vehicleStatus.lockStatus) === null || _f === void 0 ? void 0 : _f.value) === 'LOCKED')
                ? hap.Characteristic.LockTargetState.SECURED
                : hap.Characteristic.LockTargetState.UNSECURED) {
                this.log.debug('LOCK is already in the requested state');
                callback();
                return;
            }
            this.log.debug(`SET ${value ? 'Locking' : 'Unlocking'} ${accessory.displayName}`);
            let command = vehicle_1.Command.LOCK;
            if (value === hap.Characteristic.LockTargetState.UNSECURED) {
                command = vehicle_1.Command.UNLOCK;
            }
            this.pendingLockUpdate = true;
            yield this.vehicle.issueCommand(command);
            this.log.debug('Waiting 6 seconds to update vehicle info');
            yield new Promise((resolve) => setTimeout(resolve, 6000));
            this.log.debug('Done waiting...Updating vehicle info');
            yield this.vehicle.retrieveVehicleInfo();
            this.log.debug(`Lock status is now: ${(_j = (_h = (_g = this.vehicle) === null || _g === void 0 ? void 0 : _g.info) === null || _h === void 0 ? void 0 : _h.vehicleStatus.lockStatus) === null || _j === void 0 ? void 0 : _j.value}`);
            this.pendingLockUpdate = false;
            callback();
        }))
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _k, _l, _m, _o, _p;
            let lockNumber = hap.Characteristic.LockTargetState.UNSECURED;
            let lockStatus = ((_m = (_l = (_k = this.vehicle) === null || _k === void 0 ? void 0 : _k.info) === null || _l === void 0 ? void 0 : _l.vehicleStatus.lockStatus) === null || _m === void 0 ? void 0 : _m.value) || 'LOCKED';
            if (lockStatus === 'LOCKED') {
                lockNumber = hap.Characteristic.LockTargetState.SECURED;
            }
            callback(undefined, lockNumber);
            if (this.vehicle && this.vehicle.vehicleId) {
                if (!this.pendingStatusUpdate) {
                    this.pendingStatusUpdate = true;
                    yield this.vehicle.issueCommand(vehicle_1.Command.REFRESH);
                    yield new Promise((resolve) => setTimeout(resolve, 6000));
                    yield this.vehicle.retrieveVehicleInfo();
                    this.pendingStatusUpdate = false;
                    lockStatus = ((_p = (_o = this.vehicle.info) === null || _o === void 0 ? void 0 : _o.vehicleStatus.lockStatus) === null || _p === void 0 ? void 0 : _p.value) || 'LOCKED';
                    lockNumber =
                        lockStatus === 'LOCKED'
                            ? hap.Characteristic.LockTargetState.SECURED
                            : hap.Characteristic.LockTargetState.UNSECURED;
                    lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, lockNumber);
                    lockService.updateCharacteristic(hap.Characteristic.LockTargetState, lockNumber);
                }
                else {
                    const interval = setInterval(() => {
                        var _a, _b, _c;
                        if (!this.pendingStatusUpdate) {
                            clearInterval(interval);
                            lockStatus = ((_c = (_b = (_a = this.vehicle) === null || _a === void 0 ? void 0 : _a.info) === null || _b === void 0 ? void 0 : _b.vehicleStatus.lockStatus) === null || _c === void 0 ? void 0 : _c.value) || 'LOCKED';
                            lockNumber =
                                lockStatus === 'LOCKED'
                                    ? hap.Characteristic.LockTargetState.SECURED
                                    : hap.Characteristic.LockTargetState.UNSECURED;
                            lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, lockNumber);
                            lockService.updateCharacteristic(hap.Characteristic.LockTargetState, lockNumber);
                        }
                    }, 3000);
                }
            }
        }));
        switchService
            .setCharacteristic(hap.Characteristic.On, false)
            .getCharacteristic(hap.Characteristic.On)
            .on("set", (value, callback) => __awaiter(this, void 0, void 0, function* () {
            var _q, _r, _s, _t, _u, _v, _w;
            this.log.debug(`SET ${value ? 'Starting' : 'Stopping'} ${accessory.displayName} ${(_s = (_r = (_q = this.vehicle) === null || _q === void 0 ? void 0 : _q.info) === null || _r === void 0 ? void 0 : _r.vehicleStatus.lockStatus) === null || _s === void 0 ? void 0 : _s.value}`);
            if (value === (((_u = (_t = this.vehicle) === null || _t === void 0 ? void 0 : _t.info) === null || _u === void 0 ? void 0 : _u.vehicleStatus.ignitionStatus.value) === 'ON')) {
                this.log.debug('Engine is already in the requested state');
                callback();
                return;
            }
            this.log.debug(`${value ? 'Starting' : 'Stopping'} ${accessory.displayName}`);
            if (value) {
                yield this.vehicle.issueCommand(vehicle_1.Command.START);
            }
            else {
                yield this.vehicle.issueCommand(vehicle_1.Command.STOP);
            }
            yield new Promise((resolve) => setTimeout(resolve, 6000));
            yield this.vehicle.retrieveVehicleInfo();
            this.log.debug(`Start status is now: ${(_w = (_v = this.vehicle) === null || _v === void 0 ? void 0 : _v.info) === null || _w === void 0 ? void 0 : _w.vehicleStatus.ignitionStatus.value}`);
            callback();
        }))
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _x, _y, _z, _0;
            let engineStatus = ((_y = (_x = this.vehicle) === null || _x === void 0 ? void 0 : _x.info) === null || _y === void 0 ? void 0 : _y.vehicleStatus.ignitionStatus.value) || 'OFF';
            callback(undefined, engineStatus);
            if (this.vehicle && this.vehicle.vehicleId) {
                if (!this.pendingStatusUpdate) {
                    this.pendingStatusUpdate = true;
                    yield this.vehicle.issueCommand(vehicle_1.Command.REFRESH);
                    yield new Promise((resolve) => setTimeout(resolve, 6000));
                    yield this.vehicle.retrieveVehicleInfo();
                    this.pendingStatusUpdate = false;
                    engineStatus = ((_0 = (_z = this.vehicle) === null || _z === void 0 ? void 0 : _z.info) === null || _0 === void 0 ? void 0 : _0.vehicleStatus.ignitionStatus.value) || 'OFF';
                    switchService.getCharacteristic(hap.Characteristic.On).updateValue(engineStatus === 'ON');
                }
                else {
                    const interval = setInterval(() => {
                        var _a, _b;
                        if (!this.pendingStatusUpdate) {
                            clearInterval(interval);
                            engineStatus = ((_b = (_a = this.vehicle) === null || _a === void 0 ? void 0 : _a.info) === null || _b === void 0 ? void 0 : _b.vehicleStatus.ignitionStatus.value) || 'OFF';
                            switchService.getCharacteristic(hap.Characteristic.On).updateValue(engineStatus === 'ON');
                        }
                    }, 3000);
                }
            }
        }));
        batteryService
            .setCharacteristic(hap.Characteristic.BatteryLevel, 100)
            .getCharacteristic(hap.Characteristic.BatteryLevel)
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _1, _2, _3, _4, _5, _6, _7, _8, _9;
            if (!this.vehicle) {
                callback(undefined, 100);
                return;
            }
            const fuel = (_3 = (_2 = (_1 = this.vehicle) === null || _1 === void 0 ? void 0 : _1.info) === null || _2 === void 0 ? void 0 : _2.vehicleStatus.fuelLevel) === null || _3 === void 0 ? void 0 : _3.value;
            const battery = (_6 = (_5 = (_4 = this.vehicle) === null || _4 === void 0 ? void 0 : _4.info) === null || _5 === void 0 ? void 0 : _5.vehicleDetails.batteryChargeLevel) === null || _6 === void 0 ? void 0 : _6.value;
            let level = fuel || battery || 100;
            if (level > 100) {
                level = 100;
            }
            if (level < 0) {
                level = 0;
            }
            callback(undefined, level);
            const chargingStatus = (_9 = (_8 = (_7 = this.vehicle) === null || _7 === void 0 ? void 0 : _7.info) === null || _8 === void 0 ? void 0 : _8.vehicleStatus.chargingStatus) === null || _9 === void 0 ? void 0 : _9.value;
            batteryService.updateCharacteristic(hap.Characteristic.BatteryLevel, level);
            if (battery) {
                if (chargingStatus === 'ChargingAC') {
                    batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.CHARGING);
                }
                else {
                    batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.NOT_CHARGING);
                }
            }
            else {
                batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.NOT_CHARGEABLE);
            }
            if (level < 20) {
                batteryService.updateCharacteristic(hap.Characteristic.StatusLowBattery, hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW);
            }
            else {
                batteryService.updateCharacteristic(hap.Characteristic.StatusLowBattery, hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
            }
        }));
        this.accessories.push(accessory);
    }
    didFinishLaunching() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug('Configuring FordPass');
            const ford = new fordpass_connection_1.Connection(this.config, this.log, this.api);
            const authInfo = yield ford.auth();
            if (authInfo) {
                setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    this.log.debug('Reauthenticating with refresh token');
                    yield ford.getRefreshToken();
                }), ((_a = authInfo.expires_in) !== null && _a !== void 0 ? _a : 0) * 1000 - 10000);
                yield this.addVehicle(ford);
                yield this.updateVehicle();
                yield this.refreshVehicle();
                setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.updateVehicle();
                }), 60 * 1000 * 5);
            }
        });
    }
    addVehicle(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const vehicles = yield connection.getVehicles();
            if (vehicles) {
                const vehicleConfig = vehicles[0];
                vehicleConfig.vehicleId = vehicleConfig.vehicleId.toUpperCase();
                this.vehicle.vehicleId = vehicleConfig.vehicleId;
                const name = vehicleConfig.nickName || vehicleConfig.modelYear + ' ' + vehicleConfig.make + ' ' + vehicleConfig.modelName;
                const uuid = hap.uuid.generate(vehicleConfig.vehicleId);
                const accessory = new Accessory(name, uuid);
                accessory.context.name = name;
                accessory.context.vehicleId = vehicleConfig.vehicleId;
                const accessoryInformation = accessory.getService(hap.Service.AccessoryInformation);
                if (accessoryInformation) {
                    accessoryInformation.setCharacteristic(hap.Characteristic.Manufacturer, 'Ford');
                    accessoryInformation.setCharacteristic(hap.Characteristic.Model, name);
                    accessoryInformation.setCharacteristic(hap.Characteristic.SerialNumber, vehicleConfig.vehicleId);
                }
                if (!this.accessories.find((x) => x.UUID === uuid)) {
                    this.log.debug(`New vehicle found: ${name}`);
                    this.configureAccessory(accessory);
                    this.api.registerPlatformAccessories(PLUGIN_NAME, PLATFORM_NAME, [accessory]);
                }
            }
        });
    }
    updateVehicle() {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve) => setTimeout(resolve, 10000));
            yield this.vehicle.retrieveVehicleInfo();
            const status = (_b = (_a = this.vehicle) === null || _a === void 0 ? void 0 : _a.info) === null || _b === void 0 ? void 0 : _b.vehicleStatus;
            const lockStatus = (_c = status === null || status === void 0 ? void 0 : status.lockStatus) === null || _c === void 0 ? void 0 : _c.value;
            let lockNumber = hap.Characteristic.LockCurrentState.UNSECURED;
            if (lockStatus === 'LOCKED') {
                lockNumber = hap.Characteristic.LockCurrentState.SECURED;
            }
            const engineStatus = status === null || status === void 0 ? void 0 : status.ignitionStatus.value;
            let started = true;
            if (engineStatus === 'OFF') {
                started = false;
            }
            const uuid = hap.uuid.generate(this.vehicle.vehicleId);
            const accessory = this.accessories.find((x) => x.UUID === uuid);
            if (accessory) {
                const fordAccessory = new accessory_1.FordpassAccessory(accessory);
                if (!this.pendingLockUpdate) {
                    const lockService = fordAccessory.findService(hap.Service.LockMechanism);
                    lockService && lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, lockNumber);
                    lockService && lockService.updateCharacteristic(hap.Characteristic.LockTargetState, lockNumber);
                }
                const switchService = fordAccessory.findService(hap.Service.Switch);
                switchService && switchService.updateCharacteristic(hap.Characteristic.On, started);
                const plugService = fordAccessory.findService(hap.Service.OccupancySensor, 'Plug');
                plugService &&
                    plugService.updateCharacteristic(hap.Characteristic.OccupancyDetected, ((_d = status === null || status === void 0 ? void 0 : status.plugStatus) === null || _d === void 0 ? void 0 : _d.value)
                        ? hap.Characteristic.OccupancyDetected.OCCUPANCY_DETECTED
                        : hap.Characteristic.OccupancyDetected.OCCUPANCY_NOT_DETECTED);
                this.log.debug(`Charging status: ${(_e = status === null || status === void 0 ? void 0 : status.chargingStatus) === null || _e === void 0 ? void 0 : _e.value}`);
                const chargingService = fordAccessory.findService(hap.Service.OccupancySensor, 'Charging');
                chargingService &&
                    chargingService.updateCharacteristic(hap.Characteristic.OccupancyDetected, ((_f = status === null || status === void 0 ? void 0 : status.chargingStatus) === null || _f === void 0 ? void 0 : _f.value) === 'ChargingAC'
                        ? hap.Characteristic.OccupancyDetected.OCCUPANCY_DETECTED
                        : hap.Characteristic.OccupancyDetected.OCCUPANCY_NOT_DETECTED);
            }
            else {
                this.log.warn(`Accessory not found for ${this.vehicle.name}`);
            }
        });
    }
    refreshVehicle() {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug(`Configuring ${this.vehicle.name} (${this.config.autoRefresh}) to refresh every ${this.config.refreshRate} minutes.`);
            if (this.vehicle.autoRefresh && this.vehicle.refreshRate && this.vehicle.refreshRate > 0) {
                setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    this.log.debug(`Refreshing info for ${this.vehicle.name}`);
                    yield this.vehicle.issueCommand(vehicle_1.Command.REFRESH);
                }), 60000 * this.vehicle.refreshRate);
            }
        });
    }
}
module.exports = (api) => {
    hap = api.hap;
    Accessory = api.platformAccessory;
    api.registerPlatform(PLUGIN_NAME, PLATFORM_NAME, FordPassPlatform);
};
//# sourceMappingURL=index.js.map